using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebPartPages;

namespace RH.TagCloud
{
    /// <summary>
    /// Description of the toolpart. Override the GetToolParts method
    /// in your Web Part class to invoke this toolpart. To establish
    /// a reference to the Web Part the user has selected, use the
    /// ParentToolPane.SelectedWebPart property.
    /// </summary>
    public class CustomToolPart : ToolPart
    {
        // declaring variables
        private string TechnoratiUrl;
        private string List;

        // an event handler for the Init event
        private void CustomToolPart_Init(object sender, EventArgs e)
        {
            TechnoratiUrl = UniqueID + "TechnoratiUrl";
            List = UniqueID + "List";
        }

        /// <summary>
        /// Constructor for the class. A great place to set Set
        /// default values for additional base class properties
        /// here.
        /// <summary>
        public CustomToolPart()
        {
            // Set default properties
            Title = "Ren Hzser - Tag Cloud";
            Init += CustomToolPart_Init;
        }

        ///	<summary>
        ///	Called by the tool pane to apply property changes to
        /// the selected Web Part.
        ///	</summary>
        public override void ApplyChanges()
        {
            var wp1 = (TagCloud) ParentToolPane.SelectedWebPart;

            // Send the values to the Web Part.
            wp1.TechnoratiUrl = SPEncode.HtmlEncode(Page.Request.Form[TechnoratiUrl]);
            wp1.List = SPEncode.HtmlEncode(Page.Request.Form[List]);
        }

        /// <summary>
        ///	If the ApplyChanges method succeeds, this method is
        /// called by the tool pane to refresh the specified
        /// property values in the toolpart user interface.
        /// </summary>
        public override void SyncChanges()
        {
            // sync with the new property changes here
        }

        /// <summary>
        ///	Called by the tool pane if the user discards changes
        /// to the selected Web Part.
        /// </summary>
        public override void CancelChanges()
        {
        }

        /// <summary>
        /// Render this Tool part to the output parameter
        /// specified.
        /// </summary>
        /// <param name="writer">
        /// The HTML writer to write out to
        /// </param>
        protected override void RenderToolPart(HtmlTextWriter writer)
        {
            try
            {
                // Establish a reference to the Web Part.
                var wp1 = (TagCloud) ParentToolPane.SelectedWebPart;

                writer.WriteLine("Technorati Url<br/>");
                var textBox = new TextBox {ID = TechnoratiUrl, Text = wp1.TechnoratiUrl};
                textBox.RenderControl(writer);
                writer.WriteLine("<br/><br/>");
                
                writer.WriteLine("List<br/>");
                var listName = new TextBox {ID = List, Text = wp1.List};
                listName.RenderControl(writer);
                
                writer.WriteLine("</br></br>");
            }
            catch (Exception ex)
            {
                writer.Write(string.Format("Error in RenderToolPart:\r\n{0}", Common.GetErrorMessage(ex)));
            }
        }

    }
}