﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebPartPages;

namespace RH.TagCloud
{
    public class TagCloud : WebPart
    {
        private string _ErrorMessage = string.Empty;
        string _TechnoratiUrl;
        private string _List;
        private readonly Version _Version = new Version(1, 0, 1, 0);
        private bool _DesignTime;
        private Label _TagCloud;
        private IList<Tag> _Tags;
        private string _SelectedTag = string.Empty;

        #region properties
        [Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Personal), FriendlyName("Text"), Description("Technorati Url")]
        public string TechnoratiUrl
        {
            get
            {
                if (string.IsNullOrEmpty(_TechnoratiUrl))
                {
                    _TechnoratiUrl = "http://technorati.com/tag";
                }
                return _TechnoratiUrl;
            }

            set
            {
                _TechnoratiUrl = value;
            }
        }

        [Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Personal), FriendlyName("Text"), Description("List")]
        public string List
        {
            get
            {
                if (string.IsNullOrEmpty(_List))
                {
                    foreach (SPList list in SPContext.Current.Web.Lists)
                    {
                        if (list.BaseTemplate != SPListTemplateType.Posts) continue;
                        _List = list.Title;
                        break;
                    }
                }
                return _List;
            }

            set
            {
                _List = value;
            }
        }

        public Version Version
        {
            get { return _Version; }
        }
        #endregion

        public override ToolPart[] GetToolParts()
        {
            var toolparts = new ToolPart[3];
            var custom = new CustomPropertyToolPart();
            var wptp = new WebPartToolPart();
            var footer = new CustomPropertyToolPart();
            toolparts[0] = custom;
            toolparts[1] = wptp;
            toolparts[2] = footer;

            // This is the custom ToolPart.
            toolparts[0] = new CustomToolPart();
            toolparts[2] = new CustomToolPartFooter();

            return toolparts;
        }

        #region overrides
        protected override void CreateChildControls()
        {
            base.CreateChildControls();
            try
            {
                _DesignTime = DesignTime();
                if (_DesignTime) return;

                _TagCloud = new Label {ID = "TagCloudLabel"};
                Controls.Add(_TagCloud);

                CreateTagCloud();
            }
            catch (Exception ex)
            {
                _ErrorMessage += string.Format("Error in CreateChildControls:\r\n{0}\r\n", Common.GetErrorMessage(ex));
            }
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            try
            {
                Common.WriteULSVerbose("Entering RenderContents");
                if (Parent.GetType().FullName == "Microsoft.SharePoint.WebPartPages.WebPartPreview" || _DesignTime)
                {
                    writer.Write("<img src=\"/_layouts/images/RH.TagCloudPreview.jpg\" border=\"0\" alt=\"RH.TagCloudPreview Webpart from René Hézser\">");
                }
                else
                {
                    EnsureChildControls();
                    base.RenderContents(writer);

                    if (!string.IsNullOrEmpty(_ErrorMessage))
                    {
                        Common.WriteULSError(_ErrorMessage);
                        var errorControl = new HtmlGenericControl("DIV") { InnerHtml = ("<IMG src=\"/_layouts/images/error16by16.GIF\" border=\"0\"/>" + _ErrorMessage) };
                        errorControl.Attributes.Add("style", "padding: 20px; margin: 20px; border-style: groove; background-color: #FFFFFF;");
                        errorControl.RenderControl(writer);
                    }
                }
            }
            catch (Exception ex)
            {
                Common.WriteULSError(ex.ToString());
                var errorControl = new HtmlGenericControl("DIV") { InnerHtml = ("<IMG src=\"/_layouts/images/error16by16.gif\" border=\"0\"/>" + ex) };
                errorControl.Attributes.Add("style", "padding: 20px; margin: 20px; border-style: groove; background-color: #FFFFFF;");
                writer.Write(ex.ToString());
            }
        }
        #endregion

        /// <summary>
        /// create the TagCloud
        /// </summary>
        private void CreateTagCloud()
        {
            try
            {
                GetTags(true);
                if (_Tags == null) return;

                IEnumerable<IGrouping<string, Tag>> groupedTags = _Tags.GroupBy(p => p.Tagname).OrderBy(p => p.Key);
                if (groupedTags.Count() == 0) return;

                List<int> tagCounts = groupedTags.Select(t => t.Count()).Distinct().ToList();
                tagCounts.Sort();
                int minCount = tagCounts[0];
                int maxCount = tagCounts[tagCounts.Count - 1];
                int distribution = (maxCount - minCount)/3;

                for (int i = 0; i < groupedTags.Count(); i++)
                {
                    IGrouping<string, Tag> tag = groupedTags.ToArray()[i];
                    int tagCount = tag.Count();
                    string fontSize;

                    if (tagCount == minCount)
                        fontSize = "xx-small";
                    else if (tagCount == maxCount)
                        fontSize = "x-large";
                    else if (tagCount > minCount + (distribution*2))
                        fontSize = "large";
                    else if (tagCount > minCount + distribution)
                        fontSize = "medium";
                    else
                        fontSize = "small";

                    var link = new LinkButton {ID = string.Format("TagLink{0}", tag.Key), Text = tag.Key, ToolTip = tagCount.ToString(), CommandName = "Tag", CommandArgument = tag.Key};
                    link.Style.Add("font-size", fontSize);
                    link.Click += link_Click;
                    _TagCloud.Controls.Add(link);

                    // add a seperator
                    if (i != groupedTags.Count() - 1)
                    {
                        _TagCloud.Controls.Add(new Label {Text = ",&nbsp;"});
                    }
                }

                _TagCloud.Controls.Add(new Label{Text="<br/>"});
                // create refresh button
                var refresh = new ImageButton {ID = "Refresh", ImageUrl = "/_layouts/images/REFRESH.GIF", ToolTip="Refresh TagCloud"};
                refresh.Click += refresh_Click;
                _TagCloud.Controls.Add(refresh);
            }
            catch (Exception ex)
            {
                _ErrorMessage += string.Format("Error in CreateTagCloud:\r\n{0}\r\n", Common.GetErrorMessage(ex));
            }
        }

        /// <summary>
        /// get Tags from posts
        /// </summary>
        /// <param name="fromCache">if true, get data from Webpart cache</param>
        private void GetTags(bool fromCache)
        {
            try
            {
                // try to read from Webpart cache
                if (PartCacheRead(Storage.Shared, "RH.TagCloud") != null && fromCache)
                {
                    _Tags = PartCacheRead(Storage.Shared, "RH.TagCloud") as List<Tag>;
                    return;
                }


                var web = SPContext.Current.Web;
                var list = web.Lists[List];
                if (list.BaseTemplate != SPListTemplateType.Posts)
                {
                    _TagCloud.Text = "List is not a blog posts list.";
                    return;
                }

                _Tags = new List<Tag>();
                var spQuery = new SPQuery
                                  {
                                      Query = string.Format("<Where><Contains><FieldRef Name='Body' /><Value Type='Note'>{0}</Value></Contains></Where>", TechnoratiUrl)
                                  };

                SPListItemCollection results = list.GetItems(spQuery);
                if (results.Count == 0)
                {
                    _TagCloud.Text = "No Search Results Returned.";
                    return;
                }

                string dispFormUrl = string.Format("{0}/{1}?ID=", web.Url, list.Forms[PAGETYPE.PAGE_DISPLAYFORM].Url);
                foreach (SPListItem result in results)
                {
                    var body = result[SPBuiltInFieldId.Body].ToString();

                    var regex = new Regex(@"<a href="""+_TechnoratiUrl+@"(?<tag>[^""]*)""(.*?)>(?<text>.*?)</a>",
                        RegexOptions.Compiled | RegexOptions.ExplicitCapture | RegexOptions.Singleline);
                    MatchCollection matches = regex.Matches(body);

                    foreach (Match match in matches)
                    {
                        var tag = new Tag(dispFormUrl, result, match.Groups["text"].Value);
                        _Tags.Add(tag);
                    }
                }

                // write to Webpart cache
                PartCacheWrite(Storage.Shared, "RH.TagCloud", _Tags, TimeSpan.FromMinutes(5));
            }
            catch (Exception ex)
            {
                _ErrorMessage += string.Format("Error in GetTags:\r\n{0}\r\n", Common.GetErrorMessage(ex));
            }
        }

        /// <summary>
        /// create a table with posts mathing the tag
        /// </summary>
        private void CreatePostsTable()
        {
            // create a seperator
            Controls.Add(new HtmlGenericControl("HR"));

            var table = new Table();
            var headerRow = new TableHeaderRow();
            var headerCell = new TableHeaderCell {Text = _SelectedTag};
            headerRow.Cells.Add(headerCell);
            table.Rows.Add(headerRow);
            Controls.Add(table);

            var posts = _Tags.Where(t => t.Tagname == _SelectedTag);
            foreach (Tag post in posts)
            {
                var row = new TableRow();
                var cell = new TableCell {CssClass = "ms-CategoryTitleCell"};
                cell.Controls.Add(new HyperLink {Text = post.PostTitle, NavigateUrl = post.PostUrl});
                row.Cells.Add(cell);
                table.Rows.Add(row);
            }
        }

        #region Handler
        void link_Click(object sender, EventArgs e)
        {
            try
            {
                _SelectedTag = ((LinkButton)(sender)).CommandArgument;
                CreatePostsTable();
            }
            catch (Exception ex)
            {
                _ErrorMessage += string.Format("Error in link_Click:\r\n{0}\r\n", Common.GetErrorMessage(ex));
            }
        }

        void refresh_Click(object sender, ImageClickEventArgs e)
        {
            PartCacheInvalidate(Storage.Shared, "RH.TagCloud");
            GetTags(false);
        }
        #endregion

        #region helper
        public string GetDesignTimeHtml()
        {
            return "<br/><img src=\"/_layouts/images/RH.TagCloudPreview.jpg\" border=\"0\" alt=\"RH.TagCloud Webpart from René Hézser\">";
        }

        /// <summary>
        /// Are we in Design Time?
        /// </summary>
        /// <returns></returns>
        private bool DesignTime()
        {
            // if calling Page.Request fails, we are in SharePoint Designer preview.
            try
            {
#pragma warning disable 168
                var req = Page.Request;
#pragma warning restore 168
            }
            catch
            {
                return true;
            }
            return false;
        }
        #endregion
    }
}
